/*
	ibmpc/mlib.c   by <skl@van-bc.UUCP>   August/87

	Mailer UA system-dependent library

	Services to provide in mlib.c:

	Get a single character from the console.
	Invoke the local editor on a given file.
	Determine if a given file stream points to the console.
	Get a line from the console.
	Invoke a local pager on a given file.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <io.h>

#include "lib.h"
#include "host.h"

/*
	Get_One - get a single character from the console
*/

int Get_One()
{

	return getch();

} /*Get_One*/


/*
	Invoke_Editor - invoke the user's editor to edit a text file
*/

int Invoke_Editor(ecmd, filename)
char *ecmd, *filename;
{
	char command[128];

	if (ecmd == nil(char)) {
		printf("Invoke_Editor: No editor specified.\n");
		return 1;
	}

	sprintf(command, ecmd, filename);
	if (system(command) != 0) {
		printf("Invoke_Editor: system(\"%s\") failed.\n", command);
		return 2;
	}

	return 0;

} /*Invoke_Editor*/


/*
	Is_Console - is this stream from the console?
*/

int Is_Console(stream)
FILE *stream;
{

	return isatty(fileno(stream));

} /*Is_Console*/


/*
	Console_fgets - get a line of input from the local console

	This is a hook to allow for using the local system's facility
	for input line editing.  We call DOS to perform a line read,
	thus allowing utilities like DOSEDIT or CED to do their fancy work.

*/

#ifndef SIMPLE_CONSOLE_FGETS

int Console_fgets(buff, buflen, prompt)
char *buff;
int buflen;
char *prompt;
{
	static int eof = FALSE;		/* pending EOF flag	*/

	union REGS regs;
	struct SREGS sregs;

	struct {
		unsigned char maximum, actual;
		char buffer[256];
	} request;

	char *eofptr;

	if (eof) {				/* have a pending EOF?	*/
		eof = FALSE;		/* no more pending EOF	*/
		return FALSE;		/* signal the EOF		*/
	}

	if (buflen > 256)
		buflen = 256;		/* that's the most we could handle	*/

	fputs(prompt, stdout);

	request.maximum = (unsigned char)(buflen - 1);
	regs.h.ah = 0x0a;		/* buffered keyboard input	*/
	sregs.ds = _DS;
	regs.x.dx = (unsigned int)(&request);
	intdosx(&regs, &regs, &sregs);
	putchar('\n');

	/* make sure line read end with '\n\0' */
	request.buffer[request.actual] = '\n';
	request.buffer[request.actual + 1] = '\0';

	if ((eofptr = strchr(request.buffer, '\x01a')) == nil(char)) {
		strcpy(buff, request.buffer);
		return TRUE;		/* an ordinary successful read	*/
    } else if (eofptr == &request.buffer[0])
		return FALSE;		/* signal EOF right away		*/
	else {
		eof = TRUE;			/* we now have a pending EOF	*/
		*eofptr = '\0';
		strcpy(buff, request.buffer);
		return TRUE;		/* read successful but EOF next	*/
	}

} /*Console_fgets*/

#else

int Console_fgets(buff, buflen, prompt)
char *buff;
int buflen;
char *prompt;
{

	return (fgets(buff, buflen, stdin) != nil(char)) ? TRUE : FALSE;

} /*Console_fgets*/

#endif


/*
	L_invoke_pager - invoke the user's pager to view a text file
*/

int L_invoke_pager(pcmd, filename)
char *pcmd, *filename;
{
	char command[128];

	if (pcmd == nil(char)) {
		printf("L_invoke_pager: No pager specified.\n");
		return 1;
	}

	sprintf(command, pcmd, filename);
	if (system(command) != 0) {
		printf("L_invoke_pager: system(\"%s\") failed.\n", command);
		return 2;
	}

	return 0;

} /*L_invoke_pager*/